#############################################################
 # 
 # FILE:        UserAlloc.py
 # DATE:        11/10/2021
 # COMPANY:     BitFlow, Inc.
 # DESCRIPTION: A simple example of capturing data 
 #              to user allocated memory from a single frame grabber 
 #              board.
 #
#############################################################


import threading
import numpy as np

import resource

import BFModule.BufferAcquisition as Buf
import BFModule.Display as Disp

CirAq = Buf.clsCircularAcquisition()
CirAq.Open(0)


xsize = 1024; #CirAq.GetBoardInfo(BFModule.BufferAcquisition.InquireParams.XSize)
ysize = 1024; #CirAq.GetBoardInfo(BFModule.BufferAcquisition.InquireParams.YSize)

NumBuffers = 100 

#Calculate the framesize
allocSz = xsize * ysize

#Chunk to page boundary
pagesz = resource.getpagesize()
allocSz += pagesz - 1
allocSz = int( allocSz / pagesz)
allocSz *= pagesz
#For all buffers
allocSz *= NumBuffers

#Plus one page in order to align the pointer.
allocSz += pagesz

BufArray1 = np.empty([allocSz], dtype='uint32')

#Buffer Setup using the user allocated buffer array
BufArray = CirAq.BufferSetup(BufArray1, NumBuffers)
CirAq.AqSetup(0)

print("Image Size =", xsize, "X", ysize)
print("BitDepth = ", CirAq.GetBoardInfo(Buf.InquireParams.BitsPerPix))

#Create and open Display surface using the above parameters
dispSurf = Disp.clsDisp(CirAq)
#Open the display surface
dispSurf.Open()

#Start Acquisition
CirAq.AqControl(Buf.AcqCommands.Start, Buf.AcqControlOptions.Wait)
print("Acquisition Started.")

keepUpdating = True

try:
    while keepUpdating:
        try:
            if(CirAq.GetAcqStatus().Start == True):
                curBuf = CirAq.WaitForFrame(1000)
                if (dispSurf.isOpen):
                    dispSurf.Update(BufArray[curBuf.BufferNumber])
        except Exception as e:
            print(e)
            keepUpdating = False
            pass
except KeyboardInterrupt:
    pass    

CirAq.AqControl(Buf.AcqCommands.Stop, Buf.AcqControlOptions.Wait)

#Print capture stats
capStatus = CirAq.GetCaptureStatus()
print("Frames Captured = ", capStatus[0])
print("Frames Missed = ", capStatus[1])

#Cleanup acquisition
CirAq.AqCleanup()

#Close the display surface and free the resources
dispSurf.Close()

CirAq.BufferCleanup()
CirAq.Close()
